==> 0.0.13.tar.gz: lib/renderer/cache_renderer_mixin.dart <==
import 'dart:ui';

import 'package:flutter/foundation.dart';

import 'renderer.dart';

mixin CacheRendererMixin on Renderer {
  bool _waitingRefreshRender = true;
  bool _waitingRefreshChildren = true;
  Picture? _renderCache;
  Picture? _childrenCache;

  bool needCacheRender = true;
  bool needCacheChildren = true;

  @override
  void updateTree() {
    if (_waitingRefreshRender) update();
    if (_waitingRefreshChildren) updateChildren();
  }

  @override
  void renderTree(Canvas canvas) {
    final needRender = beforeRender();
    final needRenderChildren = beforeRenderChildren();

    if (needRender || needRenderChildren) {
      transformCanvas(canvas, () {
        if (needRender) _renderWithCache(canvas);
        if (needRenderChildren) _renderChildrenWithCache(canvas);
      });
    }

    _waitingRefreshRender = false;
    _waitingRefreshChildren = false;
  }

  @mustCallSuper
  @override
  void tick({bool needsPaint = true}) {
    if (_waitingRefreshRender) return;

    _waitingRefreshRender = true;
    _renderCache = null;
    super.tick(needsPaint: needsPaint);
  }

  @mustCallSuper
  @override
  void tickChildren({bool fromChild = false, bool needsPaint = true}) {
    if (_waitingRefreshChildren) return;

    _waitingRefreshChildren = true;
    _childrenCache = null;
    super.tickChildren(fromChild: fromChild, needsPaint: needsPaint);
  }

  void _renderChildrenWithCache(Canvas canvas) {
    if (_waitingRefreshChildren || !needCacheChildren) {
      return renderChildren(canvas);
    }

    canvas.drawPicture(_childrenCache ??= _record(renderChildren));
  }

  void _renderWithCache(Canvas canvas) {
    if (_waitingRefreshRender || !needCacheRender) {
      return render(canvas);
    }

    canvas.drawPicture(_renderCache ??= _record(render));
  }

  Picture _record(void Function(Canvas) callback) {
    final recorder = PictureRecorder();
    callback(Canvas(recorder));
    return recorder.endRecording();
  }
}

==> 0.0.20.tar.gz: lib/renderer/cache_renderer_mixin.dart <==
import 'dart:ui';

import 'package:flutter/foundation.dart';

import 'renderer.dart';

mixin CacheRendererMixin on Renderer {
  bool _waitingRefreshRender = true;
  bool _waitingRefreshChildren = true;
  Picture? _renderCache;
  Picture? _childrenCache;

  bool needCacheRender = true;
  bool needCacheChildren = true;

  @override
  void updateTree() {
    if (_waitingRefreshRender) update();
    if (_waitingRefreshChildren) updateChildren();
  }

  @override
  void renderTree(Canvas canvas) {
    super.renderTree(canvas);

    _waitingRefreshRender = false;
    _waitingRefreshChildren = false;
  }

  @mustCallSuper
  void tickAndRender() {
    _renderCache = null;
    _waitingRefreshRender = true;
    tick();
  }

  @mustCallSuper
  void tickAndRenderChildren() {
    _clearRenderChildren();
    tick();
  }

  @mustCallSuper
  @override
  void tick({bool fromChild = false}) {
    if (fromChild) {
      _clearRenderChildren();
    }
    super.tick(fromChild: fromChild);
  }

  void _clearRenderChildren() {
    _childrenCache = null;
    _waitingRefreshChildren = true;
  }

  @override
  @nonVirtual
  void renderChildrenWrapper(Canvas canvas) {
    if (_waitingRefreshChildren || !needCacheChildren) {
      return renderChildren(canvas);
    }

    canvas.drawPicture(_childrenCache ??= _record(renderChildren));
  }

  @override
  @nonVirtual
  void renderWrapper(Canvas canvas) {
    if (_waitingRefreshRender || !needCacheRender) {
      return render(canvas);
    }

    canvas.drawPicture(_renderCache ??= _record(render));
  }

  Picture _record(void Function(Canvas) callback) {
    final recorder = PictureRecorder();
    callback(Canvas(recorder));
    return recorder.endRecording();
  }
}
